process.env['PATH'] = process.env['PATH'] + ':' + process.env['LAMBDA_TASK_ROOT'];

const spawn = require('child_process').spawn;
const request = require('request');
const Promise = require('bluebird');
const path = require('path');

var casperUrl = 'http://localhost:9997';
var casper;


function startCasperServer() {
  var casperPath = path.join(__dirname, 'node_modules', 'casperjs', 'bin', 'casperjs');
  var childArgs = [
    path.join(__dirname, 'casperjs-server.js')
  ];
  var childOptions = {
    "env": {
      "PHANTOMJS_EXECUTABLE": path.join(__dirname, 'phantomjs')
    }
  };

  console.log('Calling casperJS: ', casperPath, childArgs, childOptions);

  try {
    casper = spawn(casperPath, childArgs, childOptions);

    casper.stdout.on('data', (data) => {
      console.log(data + '');
    });

    casper.stderr.on('data', (data) => {
      console.log('Casper error  ---:> ' + data);
    });

    casper.on('close', (code) => {
      console.log('Child process exited with code ' + code);
    });

  } catch (e) {
    console.error(e);
    process.exit(1);
  }
}

function isCasperServerReady(tried) {
  if (!tried) tried = 1;
  console.log('Checking the server ready status...');
  return new Promise((resolve, reject) => {
    setTimeout(() => {
      request(casperUrl,
        (err) => {
          if (err) {
            if (tried > 19) return reject('Casperjs server cannot be reached.'); // max retry 20 times
            return resolve(isCasperServerReady(tried + 1));
          } else {
            console.log('Casperjs server is ready');
            resolve();
          }
        });
    }, 100 * tried);
  });
}

exports.handler = function (event, context, callback) {
  console.log('Received event from handler:', JSON.stringify(event, null, 2));
  startCasperServer();
  isCasperServerReady()
  .then(() => {
    console.log('Sending request to casper server...', casperUrl);
    request.post(
      {
        headers: {'content-type': 'application/x-www-form-urlencoded'},
        url: casperUrl,
        body: "eval=" + event['eval']
      },
      (err, res, body) => {
        if (err) {
          console.log(err);
          return context.done(err);
        }
        else if (res.statusCode != 200) return context.done(body);

        context.done(null, body);

      });
  })
  .catch((err) => {
    context.done(err);
  });
};